/* Main code for Coin Flipper */
/* Copyright (c) 2002 Brian Sniffen <bts@alum.mit.edu>.
 * This work is licensed to you under the terms of the GNU General Public License, version 2.
 * A copy should be included in the file LICENSE.
 *
 * This program was written for use in the MIT Assassins' Guild game L5.
 */

#include <PalmOS.h>
#include "callback.h"
#include <stdlib.h>
#include <stdio.h>

#include "coinsRsc.h"

static int flipCoins (void);
static int updateLabel(UInt16 sliderID, UInt16 labelID, const char *display);

static Boolean MainFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    
    CALLBACK_PROLOGUE

    switch (e->eType) {
    case frmOpenEvent:
	frm = FrmGetFormPtr(e->data.frmOpen.formID);
	FrmDrawForm(frm);
	handled = true;
	break;

    case menuEvent:
	MenuEraseStatus(NULL);

	switch(e->data.menu.itemID) {
		case menuItemAbout:
			FrmPopupForm(infoForm);
			break;
	}

    	handled = true;
	break;

    case ctlSelectEvent:
	switch(e->data.ctlSelect.controlID) {
		case MainFormFlipButton:
			flipCoins();
			break;
		case MainFormCoinSlider:
			updateLabel(MainFormCoinSlider, MainFormLabel, "%hu Heads");
			break;
		case MainFormDoubleSlider:
			updateLabel(MainFormDoubleSlider, MainFormDoubleLabel, "%hu Doubles");
			break;
		case infoFormOkButton:
			FrmReturnToForm(MainForm);
		default:
			break;
	}
	break;

    default:
        break;
    }

    CALLBACK_EPILOGUE

    return handled;
}

static int flipCoins (void) {
	FormPtr frm = FrmGetActiveForm();
	UInt16 sliderIdx = FrmGetObjectIndex(frm, MainFormCoinSlider);
	ControlType *fld = FrmGetObjectPtr(frm, sliderIdx);
	UInt16 doubleIdx = FrmGetObjectIndex(frm, MainFormDoubleSlider);
	ControlType *doubleFld = FrmGetObjectPtr(frm, doubleIdx);
	UInt16 buttonIdx = FrmGetObjectIndex(frm, MainFormFlipButton);
	UInt16 n;
	UInt16 acc = 0;
	UInt16 count = 0;
	UInt16 doubling;
	UInt16 coin;
	char *str;

	if ((str = malloc(10)) == NULL)
		return 0;

	CtlGetSliderValues(fld, NULL, NULL, NULL, &n);
	CtlGetSliderValues(doubleFld, NULL, NULL, NULL, &doubling);
	
	/* Flip a fair coin until you have n heads in a row */
	while (acc < n) {
		count++;
		coin = 1+(int) (10.0*SysRandom(0)/(sysRandomMax+1.0));
		if (coin <= 5) {
			acc++;
			if (doubling > 0) {
				acc++;
				doubling--;
			}
		} else {
			acc = 0;
		}
	}

	sprintf(str,"%hu",count);

	/* Display how many flips it took you */
	/* FrmCustomAlert(ResponseAlert, str, "", ""); */

	updateLabel(MainFormCoinSlider, MainFormLabel, "Waiting");
	FrmHideObject(frm,buttonIdx);
	
	/* Now wait that many seconds and beep each time */
	while (count > 0) {
		count--;
		sprintf(str, "%hu", count);
		updateLabel(MainFormDoubleSlider, MainFormDoubleLabel, str);
		SndPlaySystemSound(sndClick);
		SysTaskDelay(SysTicksPerSecond());
	}

	free(str);
	updateLabel(MainFormCoinSlider, MainFormLabel, "%hu Heads");
	updateLabel(MainFormCoinSlider, MainFormDoubleLabel, "%hu Doubles");
	FrmShowObject(frm,buttonIdx);
	return n;
}

static int updateLabel (UInt16 sliderID, UInt16 labelID, const char *display) {
	FormPtr frm = FrmGetActiveForm();
	UInt16 sliderIdx = FrmGetObjectIndex(frm, sliderID);
	ControlType *fld = FrmGetObjectPtr(frm, sliderIdx);
	UInt16 labelIdx = FrmGetObjectIndex(frm, labelID);
	UInt16 n;
	char *str;
	
	if ((str = malloc(10)) == NULL)
	        return 0;

	CtlGetSliderValues(fld, NULL, NULL, NULL, &n);

	sprintf(str,display,n);

	FrmHideObject(frm,labelIdx);
	FrmCopyLabel(frm,labelID,str);
	FrmShowObject(frm,labelIdx);
	free(str);
	return 1;
}


static Boolean ApplicationHandleEvent(EventPtr e)
{
    FormPtr frm;
    UInt16  formId;
    Boolean handled = false;

    if (e->eType == frmLoadEvent) {
	formId = e->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	switch(formId) {
	case MainForm:
	    FrmSetEventHandler(frm, MainFormHandleEvent);
	    break;
	case infoForm:
	    FrmSetEventHandler(frm, MainFormHandleEvent);
	    break;
	}
	handled = true;
    }

    return handled;
}

/* Get preferences, open (or create) app database */
static UInt16 StartApplication(void)
{
    FrmGotoForm(MainForm);
    return 0;
}

/* Save preferences, close forms, close app database */
static void StopApplication(void)
{
    FrmSaveAllForms();
    FrmCloseAllForms();
}

/* The main event loop */
static void EventLoop(void)
{
    UInt16 err;
    EventType e;

    do {
	EvtGetEvent(&e, evtWaitForever);
	if (! SysHandleEvent (&e))
	    if (! MenuHandleEvent (NULL, &e, &err))
		if (! ApplicationHandleEvent (&e))
		    FrmDispatchEvent (&e);
    } while (e.eType != appStopEvent);
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    UInt16 err;

    if (cmd == sysAppLaunchCmdNormalLaunch) {

	err = StartApplication();
	if (err) return err;

	EventLoop();
	StopApplication();

    } else {
	return sysErrParamErr;
    }

    return 0;
}
